/**
 *******************************************************************************
 * @file    tmpm366_dmac_int.c
 * @brief   Interrupt request functions definition of DMAC
 *           for the TOSHIBA 'TMPM366' Device Series
 * @version V0.101
 * @date    2014/05/20
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 *
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "tmpm366_dmac_int.h"

volatile WorkState TxEndFlag = BUSY;

/**
  * @brief  DMA transfer end interrupt request function.
  * @param  None
  * @retval None
  */
void INTDMAC1TC_IRQHandler(void)
{
    DMAC_TxINTReq tmp = DMAC_TX_NO_REQ;
    DMAC_INTReq state;
    TxEndFlag = BUSY;

    state = DMAC_GetINTReq(DMAC_UNIT_B);

    if (state.Bit.CH1_INTReq == 1U) {
        tmp = DMAC_GetTxINTReq(DMAC_UNIT_B, DMAC_CHANNEL_1);
        if (tmp == DMAC_TX_END_REQ) {
            TxEndFlag = DMAC_GetChannelTxState(DMAC_UNIT_B, DMAC_CHANNEL_1);
            DMAC_ClearTxINTReq(DMAC_UNIT_B, DMAC_CHANNEL_1, DMAC_INT_TX_END);
        } else {
            /* Do nothing */
        }
    } else {
        /* Do nothing */
    }
}
