/**
 *******************************************************************************
 * @file    usbd_msc.h
 * @brief   MSC(BOT) Function Header for TOSHIBA 'TMPM366' Device Series
 * @version V0.202
 * @date    2014/05/20
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 *
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

#ifndef _USBD_MSC_H_
#define _USBD_MSC_H_

#include "usbd_hw.h"

/* Max Logical Unit Number */
#define USBD_MSC_BOT_MAX_LUN                (0x00U)

/* Command Status */
#define USBD_MSC_BOT_COMMAND_PASSED         (0U)
#define USBD_MSC_BOT_COMMAND_FAILED         (1U)
#define USBD_MSC_BOT_COMMAND_PHASE_ERROR    (2U)

/* Flags of requesting STALL signal */
#define USBD_MSC_BOT_REQ_STALL              true
#define USBD_MSC_BOT_NO_STALL               false

/* Class request code */
#define USBD_MSC_BOT_MASS_STORAGE_RESET     (0xFFU)
#define USBD_MSC_BOT_GET_MAX_LUN            (0xFEU)

/* definitions of Bulk-Only Mass Storage Reset */
#define USBD_MSC_BOT_MSR_VALUE              (0x0000U)
#define USBD_MSC_BOT_MSR_LENGTH             (0x0000U)

/* definitions of Get Max LUN */
#define USBD_MSC_BOT_GML_VALUE              (0x0000U)
#define USBD_MSC_BOT_GML_LENGTH             (0x0001U)

/* status of Reset Recovery */
#define USBD_MSC_BOT_RR_NO_REQUEST          (0x00U)
#define USBD_MSC_BOT_RR_REQUESTED           (0x01U)
#define USBD_MSC_BOT_RR_RCV_MSR             (0x02U)

/* CBW Buffer Size */
#define USBD_MSC_BOT_CBW_BUFFERSIZE         (64U)

#define USBD_MSC_BOT_CBW_FLAGS_DBIT         (0x80U)
#define USBD_MSC_BOT_CBW_FLAGS_D2H          (0x80U)

#define USBD_MSC_BOT_CBW_SIG                (0x43425355U)     /* = "USBC" */
#define USBD_MSC_BOT_CSW_SIG                (0x53425355U)     /* = "USBS" */

#define USBD_MSC_BOT_CBWLEN                 (31U)
#define USBD_MSC_BOT_CSWLEN                 (13U)

/* Structure of the Command Block Wrapper */
typedef struct {
    uint32_t      ulCBWSignature;
    uint32_t      ulCBWTag;
    uint32_t      ulCBWDataTransferLength;
    uint8_t       ucCBWFlag;
    uint8_t       ucCBWLUN;
    uint8_t       ucCBWCBLength;
    uint8_t       aucCBWCB[16U];
} CBW_t;

/* Structure of the Command Status Wrapper */
typedef struct {
    uint32_t      ulCSWSignature;
    uint32_t      ulCSWTag;
    uint32_t      ulCSWDataResidue;
    uint8_t       ucCSWStatus;
} CSW_t;

/* Typedef for MSC(BOT) Callback Functions */
typedef void    (*MSC_CB_ConnStat)(bool);
typedef bool    (*MSC_CB_MassStorageReset)(void);
typedef uint8_t (*MSC_CB_GetMaxLun)(void);
typedef void    (*MSC_CB_RecvCbw)(uint8_t*);
typedef void    (*MSC_CB_TrnsData)(uint32_t, uint8_t*, void*, uint16_t);
typedef void    (*MSC_CB_Notice)(void);

/* Callback Information */
typedef struct {
    MSC_CB_ConnStat                 pfnConnStat;
    MSC_CB_MassStorageReset         pfnMassStorageReset;
    MSC_CB_GetMaxLun                pfnGetMaxLun;
    MSC_CB_RecvCbw                  pfnRecvCbw;
    MSC_CB_TrnsData                 pfnSendData;
    MSC_CB_TrnsData                 pfnRecvData;
    MSC_CB_Notice                   pfnResume;
    MSC_CB_Notice                   pfnSuspend;
} MSC_CB_t;

/* MSC(BOT) Initialize */
extern void usbd_msc_init(MSC_CB_t*);

/* MSC(BOT) Request Handler */
extern bool usbd_interrupt_MSC_req(uint32_t);

/* Other Prototypes */
extern bool USBD_MSC_SendData(uint32_t, uint8_t*, void*);
extern bool USBD_MSC_RecvData(uint32_t, uint8_t*, void*);
extern bool USBD_MSC_SetResponse(uint8_t, uint32_t, bool);
extern void USBD_MSC_CB_ClearFeature(USBD_EPx, uint16_t);
extern void USBD_MSC_Connect(void);
extern void USBD_MSC_DisConnect(void);

#endif /* _USBD_MSC_H_ */
