/**
 *******************************************************************************
 * @file    usbd_hw_com.c
 * @brief   USB H/W Common Code for TOSHIBA 'TMPM366' Device Series
 * @version V0.201
 * @date    2014/05/20
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 *
 * (C)Copyright TOSHIBA ELECTRONIC DEVICES & STORAGE CORPORATION 2017 All rights reserved
 *******************************************************************************
 */

#include "usbd_hw.h"
#include "tmpm366_gpio.h"

/**
  * @brief Software delay loop
  */
#define SOFTWARE_DELAY  0x20000U;

/**
  * @brief  Wait 1ms
  * @param  None
  * @retval None
  */
void usbd_wait_1ms(void)
{
    uint32_t delay = SOFTWARE_DELAY;
    while (delay) {
        delay--;
    }
    return;
}

/**
  * @brief  Config the clock and power of USBD
  * @param  None
  * @retval None
  */
void Config_USB(void)
{
    USBD_PowerCtrl pwr = { 0U };

    TSB_CG->USBCTL = 0x100U;    /* enable USB Clock */

    /*USBON pin config*/
//  USBPON_FR |= USBPON;
//  USBPON_IE |= USBPON;
    GPIO_SetInput(GPIO_PG, GPIO_BIT_5);
    GPIO_SetInputEnableReg(GPIO_PG, GPIO_BIT_5, ENABLE);

    /* pin PE4 control the pull up of D+, must be set output '1' */
    GPIO_SetOutput(GPIO_PE, GPIO_BIT_4);
    GPIO_WriteDataBit(GPIO_PE, GPIO_BIT_4, GPIO_BIT_VALUE_1);

    USBD_SetINTMask(USBD_INT_USB_RESET_END, ENABLE);
    USBD_SetINTMask(USBD_INT_USB_RESET, ENABLE);

    /*  UDPWCTL Power Reset and            */
    /*      PHY Reset & Suspend: 1ms       */
    pwr = USBD_GetPowerCtrlStatus();
    pwr.Bit.PW_Resetb = 0U;
    pwr.Bit.PHY_Resetb = 0U;
    pwr.Bit.PHY_Suspend = 1U;
    USBD_SetPowerCtrl(pwr);
    usbd_wait_1ms();

    /* PHY Reset off : 1ms                  */
    pwr = USBD_GetPowerCtrlStatus();
    pwr.Bit.PHY_Resetb = 1U;
    pwr.Bit.PHY_Suspend = 1U;
    USBD_SetPowerCtrl(pwr);
    usbd_wait_1ms();

    /*  PHY Suspend off : 1ms              */
    pwr = USBD_GetPowerCtrlStatus();
    pwr.Bit.PHY_Suspend = 0U;
    USBD_SetPowerCtrl(pwr);
    usbd_wait_1ms();

    usbd_wait_1ms();

    /*  UDPWCTL Power Reset Off: 1ms        */
    pwr = USBD_GetPowerCtrlStatus();
    pwr.Bit.PW_Resetb = 1U;
    USBD_SetPowerCtrl(pwr);
    usbd_wait_1ms();
    usbd_wait_1ms();
    pwr = USBD_GetPowerCtrlStatus();

    /* clear pending UDC2 interrupt and disable INT for SOF and NAK */
    USBD_WriteUDC2Reg(UDC2_INT, 0x90FFU);

    USBD_SetEPCMD(USBD_EP0, USBD_CMD_ALL_EP_INVALID);
    USBD_SetEPCMD(USBD_EP0, USBD_CMD_USB_READY);

    NVIC_EnableIRQ(INTUSB_IRQn);

// VBUS
    NVIC_EnableIRQ(INTUSBPON_IRQn);

    return;
}

/* EOC */
